@@ -19,6 +19,8 @@
 #
 # Urwid web site: http://excess.org/urwid/
 
+from __future__ import division, print_function
+
 """
 Curses-based UI implementation
 """
@@ -30,7 +32,7 @@ from urwid import escape
 
 from urwid.display_common import BaseScreen, RealTerminal, AttrSpec, \
     UNPRINTABLE_TRANS_TABLE
-from urwid.compat import bytes, PYTHON3
+from urwid.compat import bytes, PYTHON3, text_type, xrange, ord2
 
 KEY_RESIZE = 410 # curses.KEY_RESIZE (sometimes not defined)
 KEY_MOUSE = 409 # curses.KEY_MOUSE
@@ -366,7 +368,7 @@ class Screen(BaseScreen, RealTerminal):
         l = []
         def append_button( b ):
             b |= mod
-            l.extend([ 27, ord('['), ord('M'), b+32, x+33, y+33 ])
+            l.extend([ 27, ord2('['), ord2('M'), b+32, x+33, y+33 ])
 
         if bstate & curses.BUTTON1_PRESSED and last & 1 == 0:
             append_button( 0 )
@@ -481,10 +483,12 @@ class Screen(BaseScreen, RealTerminal):
 
         self.s.attrset(attr)
 
-    def draw_screen(self, (cols, rows), r ):
+    def draw_screen(self, size, r ):
         """Paint screen with rendered canvas."""
         assert self._started
 
+        cols, rows = size
+
         assert r.rows() == rows, "canvas size and passed size don't match"
 
         y = -1
@@ -511,8 +515,7 @@ class Screen(BaseScreen, RealTerminal):
                 try:
                     if cs in ("0", "U"):
                         for i in range(len(seg)):
-                            self.s.addch( 0x400000 +
-                                ord(seg[i]) )
+                            self.s.addch( 0x400000 + ord2(seg[i]) )
                     else:
                         assert cs is None
                         if PYTHON3:
@@ -558,7 +561,7 @@ class Screen(BaseScreen, RealTerminal):
 class _test:
     def __init__(self):
         self.ui = Screen()
-        self.l = _curses_colours.keys()
+        self.l = list(_curses_colours.keys())
         self.l.sort()
         for c in self.l:
             self.ui.register_palette( [
@@ -602,7 +605,7 @@ class _test:
             t = ""
             a = []
             for k in keys:
-                if type(k) == unicode: k = k.encode("utf-8")
+                if type(k) == text_type: k = k.encode("utf-8")
                 t += "'"+k + "' "
                 a += [(None,1), ('yellow on dark blue',len(k)),
                     (None,2)]
