@@ -20,8 +20,9 @@
 #
 # Urwid web site: http://excess.org/urwid/
 
+from __future__ import division, print_function
 
-from display_common import BaseScreen
+from .display_common import BaseScreen
 
 import time
 
@@ -39,7 +40,7 @@ class LCDScreen(BaseScreen):
     def reset_default_terminal_palette(self, *args):
         pass
 
-    def draw_screen(self, (cols, rows), r ):
+    def draw_screen(self, size, r ):
         pass
 
     def clear(self):
