@@ -20,6 +20,8 @@
 #
 # Urwid web site: http://excess.org/urwid/
 
+from __future__ import division, print_function
+
 import os
 import sys
 import time
@@ -43,8 +45,9 @@ from urwid.escape import DEC_SPECIAL_CHARS, ALT_DEC_SPECIAL_CHARS
 from urwid.canvas import Canvas
 from urwid.widget import Widget, BOX
 from urwid.display_common import AttrSpec, RealTerminal, _BASIC_COLORS
-from urwid.compat import ord2, chr2, B, bytes, PYTHON3
+from urwid.compat import ord2, chr2, B, bytes, PYTHON3, xrange
 
+EOF = B('')
 ESC = chr(27)
 
 KEY_TRANSLATIONS = {
@@ -244,6 +247,7 @@ class TermCanvas(Canvas):
         self.width, self.height = width, height
         self.widget = widget
         self.modes = widget.term_modes
+        self.has_focus = False
 
         self.scrollback_buffer = TermScroller()
         self.scrolling_up = 0
@@ -267,7 +271,9 @@ class TermCanvas(Canvas):
 
         self.term_cursor = self.constrain_coords(x, y)
 
-        if self.modes.visible_cursor and self.scrolling_up < self.height - y:
+        if self.has_focus \
+                and self.modes.visible_cursor \
+                and self.scrolling_up < self.height - y:
             self.cursor = (x, y + self.scrolling_up)
         else:
             self.cursor = None
@@ -671,7 +677,7 @@ class TermCanvas(Canvas):
             self.widget.beep()
         elif not dc and char in B("\x18\x1a"): # CAN/SUB
             self.leave_escape()
-        elif not dc and char == B("\x7f"): # DEL
+        elif not dc and char in B("\x00\x7f"): # NUL/DEL
             pass # this is ignored
         elif self.within_escape:
             self.parse_escape(char)
@@ -1316,47 +1322,55 @@ class TermCanvas(Canvas):
             return [self.cols()]*self.rows()
         return self.content()
 
+
 class Terminal(Widget):
     _selectable = True
     _sizing = frozenset([BOX])
 
     signals = ['closed', 'beep', 'leds', 'title']
 
-    def __init__(self, command, env=None, main_loop=None, escape_sequence=None):
+    def __init__(self, command, env=None, main_loop=None, escape_sequence=None,
+                 encoding='utf-8'):
         """
         A terminal emulator within a widget.
 
-        'command' is the command to execute inside the terminal, provided as a
+        ``command`` is the command to execute inside the terminal, provided as a
         list of the command followed by its arguments.  If 'command' is None,
         the command is the current user's shell. You can also provide a callable
         instead of a command, which will be executed in the subprocess.
 
-        'env' can be used to pass custom environment variables. If omitted,
+        ``env`` can be used to pass custom environment variables. If omitted,
         os.environ is used.
 
-        'main_loop' should be provided, because the canvas state machine needs
+        ``main_loop`` should be provided, because the canvas state machine needs
         to act on input from the PTY master device. This object must have
         watch_file and remove_watch_file methods.
 
-        'escape_sequence' is the urwid key symbol which should be used to break
-        out of the terminal widget. If it's not specified, "ctrl a" is used.
+        ``escape_sequence`` is the urwid key symbol which should be used to break
+        out of the terminal widget. If it's not specified, ``ctrl a`` is used.
+
+        ``encoding`` specifies the encoding that is being used when local
+        keypresses in Unicode are encoded into raw bytes. UTF-8 is used by default.
+        Set this to the encoding of your terminal if you need to transmit
+        characters to the spawned process in non-UTF8 encoding.
+        Applies to Python 3.x only.
+
+        .. note::
+
+            If you notice your Terminal instance is not printing unicode glyphs
+            correctly, make sure the global encoding for urwid is set to
+            ``utf8`` with ``urwid.set_encoding("utf8")``. See
+            :ref:`text-encodings` for more details.
         """
-        self.__super.__init__()
+        Widget.__init__(self)
 
-        if escape_sequence is None:
-            self.escape_sequence = "ctrl a"
-        else:
-            self.escape_sequence = escape_sequence
+        self.escape_sequence = escape_sequence or "ctrl a"
 
-        if env is None:
-            self.env = dict(os.environ)
-        else:
-            self.env = dict(env)
+        self.env = dict(env or os.environ)
 
-        if command is None:
-            self.command = [self.env.get('SHELL', '/bin/sh')]
-        else:
-            self.command = command
+        self.command = command or [self.env.get('SHELL', '/bin/sh')]
+
+        self.encoding = encoding
 
         self.keygrab = False
         self.last_key = None
@@ -1376,6 +1390,28 @@ class Terminal(Widget):
         self.has_focus = False
         self.terminated = False
 
+    def get_cursor_coords(self, size):
+        """Return the cursor coordinates for this terminal
+        """
+        if self.term is None:
+            return None
+
+        # temporarily set width/height to figure out the new cursor position
+        # given the provided width/height
+        orig_width, orig_height = self.term.width, self.term.height
+
+        self.term.width = size[0]
+        self.term.height = size[1]
+
+        x, y = self.term.constrain_coords(
+            self.term.term_cursor[0],
+            self.term.term_cursor[1],
+        )
+
+        self.term.width, self.term.height = orig_width, orig_height
+
+        return (x, y)
+
     def spawn(self):
         env = self.env
         env['TERM'] = 'linux'
@@ -1479,16 +1515,21 @@ class Terminal(Widget):
         """
         Ignore SIGINT if this widget has focus.
         """
-        if self.terminated or self.has_focus == has_focus:
+        if self.terminated:
             return
 
         self.has_focus = has_focus
 
+        if self.term is not None:
+            self.term.has_focus = has_focus
+            self.term.set_term_cursor()
+
         if has_focus:
             self.old_tios = RealTerminal().tty_signal_keys()
             RealTerminal().tty_signal_keys(*(['undefined'] * 5))
         else:
-            RealTerminal().tty_signal_keys(*self.old_tios)
+            if hasattr(self, "old_tios"):
+                RealTerminal().tty_signal_keys(*self.old_tios)
 
     def render(self, size, focus=False):
         if not self.terminated:
@@ -1505,18 +1546,13 @@ class Terminal(Widget):
     def add_watch(self):
         if self.main_loop is None:
             return
-
         self.main_loop.watch_file(self.master, self.feed)
 
     def remove_watch(self):
         if self.main_loop is None:
             return
-
         self.main_loop.remove_watch_file(self.master)
 
-    def selectable(self):
-        return True
-
     def wait_and_feed(self, timeout=1.0):
         while True:
             try:
@@ -1528,19 +1564,19 @@ class Terminal(Widget):
         self.feed()
 
     def feed(self):
-        data = ''
+        data = EOF
 
         try:
             data = os.read(self.master, 4096)
         except OSError as e:
-            if e.errno == 5: # End Of File
-                data = ''
+            if e.errno == 5: # EIO, child terminated
+                data = EOF
             elif e.errno == errno.EWOULDBLOCK: # empty buffer
                 return
             else:
                 raise
 
-        if data == '': # EOF on BSD
+        if data == EOF:
             self.terminate()
             self._emit('closed')
             return
@@ -1621,6 +1657,6 @@ class Terminal(Widget):
             key += "\x0a"
 
         if PYTHON3:
-            key = key.encode('ascii')
+            key = key.encode(self.encoding, 'ignore')
 
         os.write(self.master, key)
