@@ -19,6 +19,8 @@
 #
 # Urwid web site: http://excess.org/urwid/
 
+from __future__ import division, print_function
+
 from urwid.widget import (Text, WidgetWrap, delegate_to_widget_mixin, BOX,
     FLOW)
 from urwid.canvas import CompositeCanvas
@@ -32,8 +34,9 @@ from urwid.decoration import WidgetDecoration
 from urwid.command_map import ACTIVATE
 
 class SelectableIcon(Text):
+    ignore_focus = False
     _selectable = True
-    def __init__(self, text, cursor_position=1):
+    def __init__(self, text, cursor_position=0):
         """
         :param text: markup for this widget; see :class:`Text` for
                      description of text markup
@@ -56,7 +59,7 @@ class SelectableIcon(Text):
         >>> si
         <SelectableIcon selectable flow widget '[!]'>
         >>> si.render((4,), focus=True).cursor
-        (1, 0)
+        (0, 0)
         >>> si = SelectableIcon("((*))", 2)
         >>> si.render((8,), focus=True).cursor
         (2, 0)
@@ -101,18 +104,18 @@ class CheckBox(WidgetWrap):
         return frozenset([FLOW])
 
     states = {
-        True: SelectableIcon("[X]"),
-        False: SelectableIcon("[ ]"),
-        'mixed': SelectableIcon("[#]") }
+        True: SelectableIcon("[X]", 1),
+        False: SelectableIcon("[ ]", 1),
+        'mixed': SelectableIcon("[#]", 1) }
     reserve_columns = 4
 
     # allow users of this class to listen for change events
     # sent when the state of this widget is modified
     # (this variable is picked up by the MetaSignals metaclass)
-    signals = ["change"]
+    signals = ["change", 'postchange']
 
     def __init__(self, label, state=False, has_mixed=False,
-             on_state_change=None, user_data=None):
+             on_state_change=None, user_data=None, checked_symbol=None):
         """
         :param label: markup for check box label
         :param state: False, True or "mixed"
@@ -121,7 +124,7 @@ class CheckBox(WidgetWrap):
                                 function call for a single callback
         :param user_data: user_data for on_state_change
 
-        Signals supported: ``'change'``
+        Signals supported: ``'change'``, ``"postchange"``
 
         Register signal handler with::
 
@@ -146,6 +149,8 @@ class CheckBox(WidgetWrap):
         self._label = Text("")
         self.has_mixed = has_mixed
         self._state = None
+        if checked_symbol:
+            self.states[True] = SelectableIcon(u"[%s]" % checked_symbol, 1)
         # The old way of listening for a change was to pass the callback
         # in to the constructor.  Just convert it to the new way:
         if on_state_change:
@@ -184,12 +189,12 @@ class CheckBox(WidgetWrap):
         Return label text.
 
         >>> cb = CheckBox(u"Seriously")
-        >>> print cb.get_label()
+        >>> print(cb.get_label())
         Seriously
-        >>> print cb.label
+        >>> print(cb.label)
         Seriously
         >>> cb.set_label([('bright_attr', u"flashy"), u" normal"])
-        >>> print cb.label  #  only text is returned
+        >>> print(cb.label)  #  only text is returned
         flashy normal
         """
         return self._label.text
@@ -200,7 +205,7 @@ class CheckBox(WidgetWrap):
         Set the CheckBox state.
 
         state -- True, False or "mixed"
-        do_callback -- False to supress signal from this change
+        do_callback -- False to suppress signal from this change
 
         >>> changes = []
         >>> def callback_a(cb, state, user_data):
@@ -233,7 +238,8 @@ class CheckBox(WidgetWrap):
 
         # self._state is None is a special case when the CheckBox
         # has just been created
-        if do_callback and self._state is not None:
+        old_state = self._state
+        if do_callback and old_state is not None:
             self._emit('change', state)
         self._state = state
         # rebuild the display widget with the new state
@@ -241,6 +247,8 @@ class CheckBox(WidgetWrap):
             ('fixed', self.reserve_columns, self.states[state] ),
             self._label ] )
         self._w.focus_col = 0
+        if do_callback and old_state is not None:
+            self._emit('postchange', old_state)
 
     def get_state(self):
         """Return the state of the checkbox."""
@@ -317,9 +325,9 @@ class CheckBox(WidgetWrap):
 
 class RadioButton(CheckBox):
     states = {
-        True: SelectableIcon("(X)"),
-        False: SelectableIcon("( )"),
-        'mixed': SelectableIcon("(#)") }
+        True: SelectableIcon("(X)", 1),
+        False: SelectableIcon("( )", 1),
+        'mixed': SelectableIcon("(#)", 1) }
     reserve_columns = 4
 
     def __init__(self, group, label, state="first True",
@@ -335,7 +343,7 @@ class RadioButton(CheckBox):
         This function will append the new radio button to group.
         "first True" will set to True if group is empty.
 
-        Signals supported: ``'change'``
+        Signals supported: ``'change'``, ``"postchange"``
 
         Register signal handler with::
 
@@ -374,7 +382,7 @@ class RadioButton(CheckBox):
 
         state -- True, False or "mixed"
 
-        do_callback -- False to supress signal from this change
+        do_callback -- False to suppress signal from this change
 
         If state is True all other radio buttons in the same button
         group will be set to False.
@@ -504,9 +512,9 @@ class Button(WidgetWrap):
         Return label text.
 
         >>> b = Button(u"Ok")
-        >>> print b.get_label()
+        >>> print(b.get_label())
         Ok
-        >>> print b.label
+        >>> print(b.label)
         Ok
         """
         return self._label.text
@@ -566,7 +574,7 @@ class PopUpLauncher(delegate_to_widget_mixin('_original_widget'),
 
     def create_pop_up(self):
         """
-        Subclass must override this method and have is return a widget
+        Subclass must override this method and return a widget
         to be used for the pop-up.  This method is called once each time
         the pop-up is opened.
         """
